class Git:
    # Initiate Class
    def __init__(self, repoOwner: str, token: str = None):
        """
        Go to github account
        Token is only required if you are getting private repositories
        """
        self.REPO_OWNER = repoOwner
        self.TOKEN = token

    # Import all child methods
    from .repositories import get_repository, get_sub_repository, build_repository


import requests
import time

# return types
json = 0
str = 1

# rates
warning_limit = 75 # limit requests until warning
rate_limit = 100 # limit requests | so you dont get rate limited by github
rate_time = 5 # sec
last = time.time()
i = 0

def request(apiUrl: str, _response: int = 0, headers: set=None) -> set:
    """
    request with no python errors
    """
    # globals
    global i
    global last

    if i > warning_limit and time.time()-last < rate_limit:
        if i > rate_limit:
            print("Rate limit exceeded")
            return
        
        print("Warning: you should slowdown your requests")
    elif time.time()-last > rate_limit:
        i = 0

    try:
        response = requests.get(apiUrl, headers=headers)
        i+=1
        last = time.time()

        if response.status_code == 200:
            repo_data = response.content

            if _response == 0:
                repo_data = response.json()

            return repo_data
        else:
            print(f"Error: {response.status_code} | {response.text}")

    except Exception as e:
        print(f"Error: {str(e)}")


from gitor import api, _set, gitrequest

def get_repository_raw(self, repo: set):
    return gitrequest.request(repo[api.raw], gitrequest.str, api.get_headers(self.TOKEN))
def get_repository_JSON(self, repo: set):
    return gitrequest.request(repo[api.children], gitrequest.json, api.get_headers(self.TOKEN))

def get_repository(self, name: str, raw: bool = False) -> set or str:
    """
    get a repository
    """
    # if raw wanted then return the raw str
    if raw:
        JSON = gitrequest.request(api.get_repo_url(self.REPO_OWNER, name), gitrequest.json, api.get_headers(self.TOKEN))
        return get_repository_raw(self, JSON)
    
    return gitrequest.request(api.get_repo_url(self.REPO_OWNER, name), gitrequest.json, api.get_headers(self.TOKEN))

def get_sub_repository(self, name: str, directory: str, raw: bool = False) -> set or str:
    DirectoryList = directory.split("/")
    LastRepo = get_repository(self, name)

    if not LastRepo: return
    
    # iterate through all the direcotries
    for i in DirectoryList:
        repo = _set.find_value_in_json(LastRepo, api.name, i)
        if not repo: return
        
        # check if another url is needed
        LastRepo = gitrequest.request(repo[api.children], gitrequest.json, api.get_headers(self.TOKEN))
        if not LastRepo: print (f"Error: could not get repo {repo}"); return

    # if raw wanted then return the raw str
    if raw:
        JSON = gitrequest.request(api.get_repo_url(self.REPO_OWNER, name), gitrequest.json, api.get_headers(self.TOKEN))
        return get_repository_raw(self, JSON)
    
    return LastRepo

def build(repo_name: str, content: str):
    return {'name': repo_name, 'content': content}

def build_repository(self, repository: set) -> set:
    repo_build = {}

    def add_directory(path: str):
        directory_list = path.split("/")
        proxy = repo_build
        last = proxy
        index = 0
        
        # iterate through all the paths
        for i in directory_list:
            #print(i)
            try:
                last = last[i]
            except:
                last[i] = {}

            index+=1

        #print(proxy, last)

    def add_to_directory(build: set, path: str):
        directory_list = path.split("/")
        last = repo_build
        #proxy = last
        index = 0

        # iterate through all the paths
        for i in directory_list:
            index+=1
            if index == len(directory_list):
               last[i] = build
            else:
               try:
                    if last[i]:
                        last = last[i]
               except:
                   last[i] = {}
                   last = last[i]

    def iterate(rep: set):
        for repo in rep:
            if api.is_repo_file(repo):
                #print(repo[api.raw])
                content = get_repository_raw(self, repo)
                
                #repo_build[repo[api.name]] = build(repo[api.name],content[api.raw])
                #print(build(repo[api.name], content))
                add_to_directory(build(repo[api.name], content), repo[api.path])
            else:
                iterate(get_repository_JSON(self,repo))

    iterate(repository)
    return repo_build


name = "name"
raw = "download_url"
type = "type"
path = "path"
children = "url"

def get_repo_url(owner: str, name: str):
    """
    Get the repository url
    """
    owner = owner.lower()
    name = name.lower()
    return f"https://api.github.com/repos/{owner}/{name}/contents"

def get_headers(token: str):
    """
    get headers for private repositories
    """
    return {
        "Authorization": f"token {token}",
        "Accept": "application/vnd.github.v3+json"
    }

def is_file(type: str):
    """
    Check if the current argument value is equal to file
    """
    if type == "file": return True
    return False

def is_repo_file(repo: set):
    """
    Check if repo is a file or a directory/folder
    """
    return is_file(repo[type])


def find_value_in_json(JSON: set, attribute: str, value: str):
    """
    Find the set that has attribute that is equal to value
    """
    for i in JSON:
        if i[attribute] == value:
            return i
        
    print(f"Error: could not find {attribute} - {value}")